#!/usr/bin/env python
# epgdb-translate v0.1
# (c) cikaz 2015

import sys, os, io
import re, socket, time, urllib
from urllib2 import Request, urlopen, URLError, HTTPError
from urlparse import parse_qs
import HTMLParser
from shutil import copy2 as fcopy

import sqlite3
from sqlite3 import dbapi2 as sqlite


##############################################################################

"""
http://data.okfn.org/data/core/language-codes/r/language-codes-3b2.json
"""

TARGET_LANG='en'
LANGS_EXCLUDE = ('deu', 'ger', 'eng')
# try avoid flooding google :)
DELAY_SECS=0.5
RESTART_GUI=True
DB_SRC='/etc/enigma2/epg.db'
DB_TMP='/tmp/epg.db'

##############################################################################

class Translator(object):
	def __init__(self, dstlang, srclang='auto'):
		self.srclang = srclang
		self.dstlang = dstlang

	def translate(self, text):
		raise Exception("Implementation required")

class GTranslator(Translator):
	def __init__(self, dstlang, srclang='auto'):
		super(GTranslator, self).__init__(dstlang, srclang)

		self.url = 'http://translate.google.com/m?'
		self.parser = HTMLParser.HTMLParser()
		self.rx_unescape = re.compile('(&.+?;)')
		# <div dir="ltr" class="t0">TRANSLATED TEXT</div>
		self.tok = 'class="t0">'

	def translate(self, text):
		if not text or len(text) == 0:
			return

		_text = text

		headers = {
			'User-Agent': 'Linux / Chrome 32: Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/32.0.1700.107 Safari/537.36',
			'Accept': 'text/plain',
			'Accept-Encoding': 'utf-8',
			'Accept-Language': self.dstlang
		}
		
		values = {
			'ie' : 'UTF-8',
			'sl' : self.srclang,
			'tl' : self.dstlang,
			'hl' : self.dstlang,
			'q'  : _text,
		}
		qs = urllib.urlencode(values, True)
		url = self.url + qs
		request = Request(url, headers=headers)
		error = None
		try:
			resp = urlopen(request, timeout=20)
			data = resp.read().decode('utf-8')
			translated = data[data.find(self.tok) + len(self.tok):]
			translated = translated.split('<', 1)[0]
			translated = self.rx_unescape.sub(lambda m: self.parser.unescape(m.group()), translated)

		except URLError as e:
			error = 'URL Error: %s' % str(e.reason)
		except HTTPError as e:
			error = 'HTTP Error: %s' % str(e.code)
		except socket.error as e:
			error = 'Socket Error: %s' % str(e)
		except Exception as e:
			error = 'Error: ' + str(e)

		if not error:
			return translated
	
		print(error, url)
	

class EPGDBEvent(object):
	def __init__(self, id, lang, title='', sdescr='', edescr=''):
		self.id = id
		self.lang = lang
		self.title = title
		self.sdescr = sdescr
		self.edescr = edescr
		
	def __str__(self):
		return str((self.id, self.lang, self.title, self.sdescr, self.edescr))


class EPGDB(object):
	
	def __init__(self, epgdb_path = 'epg.db'):
		self.epgdb_path = epgdb_path
		self.conn = sqlite.connect(self.epgdb_path, timeout=20, isolation_level=None, check_same_thread=False)
		self.conn.text_factory = str
		self.conn.row_factory = sqlite3.Row

	def initTableTranslated(self):
		cmd = """
		CREATE TABLE IF NOT EXISTS T_Translated (
				id INTEGER PRIMARY KEY UNIQUE, 
				lang TEXT NOT NULL,
				changed DATETIME NOT NULL DEFAULT current_timestamp);
		"""
		c = self.execute(cmd)
		assert(c.rowcount == 1)
		
		cmd = """
		CREATE TRIGGER IF NOT EXISTS tr_on_delete_cascade_t_data_t_translated 
			AFTER DELETE ON T_Data FOR EACH ROW WHEN (
				(SELECT event_id FROM T_Data WHERE event_id = OLD.event_id LIMIT 1) ISNULL) 
				BEGIN DELETE FROM T_Translated WHERE event_id = OLD.event_id; END;
		"""
		c = self.execute(cmd)
		assert(c.rowcount == 1)

		self.conn.commit()

	def execute(self, cmd, *args):
		try:
			conn = self.conn
			with conn:
				return conn.execute(cmd, args)
		except sqlite3.Error as e:
			print('Error ' + str(e), cmd)
			raise
	
	def getLangs(self):
		c = db.execute('SELECT DISTINCT iso_639_language_code FROM T_Data')
		r = c.fetchone()
		langs = set()
		while r:
			langs.add(r[0].lower())
			r = c.fetchone()
		return list(langs)
	
	def setEvent(self, event):

		"""
		UPDATE closure SET checked = 0 
		WHERE item_id IN (SELECT id FROM item WHERE ancestor_id = 1);
		"""
		cmd = """
			SELECT * FROM T_Data
				WHERE event_id=?;
			"""
			
		c = self.execute(cmd, event.id)
		r = c.fetchone()

		cmd = """
			UPDATE T_Title
				SET title=?
				WHERE id=?;
			"""

		if event.title and len(event.title):
			c = self.execute(cmd, event.title, r['title_id'])
			assert(c.rowcount == 1)
			
		cmd = """
			UPDATE T_Short_Description
				SET short_description=?
				WHERE id=?;
			"""
		if event.sdescr and len(event.sdescr):
			c = self.execute(cmd, event.sdescr, r['short_description_id'])
			assert(c.rowcount == 1)

		cmd = """
			UPDATE T_Extended_Description
				SET extended_description=?
				WHERE id=?
			"""
		if event.edescr and len(event.edescr):
			c = self.execute(cmd, event.edescr, r['extended_description_id'])
			assert(c.rowcount == 1)

		# doesn't work, iso_639_language_code gets updated by e2 -> add translated table
		#cmd = """
		#	UPDATE T_DATA
		#		SET iso_639_language_code=?
		#		WHERE event_id=?
		#	"""
		#c = self.execute(cmd, 'ger', r['event_id'])
		#assert(c.rowcount == 1)
		cmd = """
			INSERT OR REPLACE INTO T_Translated values(?, ?, ?)
			"""
		c = self.execute(cmd, r['event_id'], event.lang, int(time.time()))
		assert(c.rowcount == 1)

		self.conn.commit()

	def getEvent(self, event_id):
		cmd = """
			SELECT iso_639_language_code FROM T_Data AS t1 
   				WHERE t1.event_id=?
    		"""
		c = self.execute(cmd, event_id)
		try:
			lang = c.fetchone()[0]
		except:
			lang = 'auto'
			pass

		cmd = """
			SELECT title FROM T_Data AS t1 
    			JOIN T_Title AS t2 ON t1.title_id=t2.id 
    			WHERE t1.event_id=?
    		"""
		c = self.execute(cmd, event_id)
		try:
			title = c.fetchone()[0]
		except:
			title = None

		cmd = """
			SELECT short_description FROM T_Data AS t1 
    			JOIN T_Short_Description AS t2 ON t1.short_description_id=t2.id 
    			WHERE t1.event_id=?
    		"""
		c = self.execute(cmd, event_id)
		try:
			sdescr = c.fetchone()[0]
		except:
			sdescr = None

		cmd = """
			SELECT extended_description FROM T_Data AS t1 
			    JOIN T_Extended_Description AS t2 ON t1.extended_description_id=t2.id 
    			WHERE t1.event_id=?
    		"""
		c = self.execute(cmd, event_id)
		try:
			edescr = c.fetchone()[0]
		except:
			edescr = None
		
		return EPGDBEvent(event_id, lang.lower(), title, sdescr, edescr)
	
	def markTranslated(self, lang, exclude_lang=()):
		q = 'SELECT * FROM T_Data'
		if len(exclude_lang):
			x = "?, " * len(exclude_lang)
			q += ' WHERE LOWER(iso_639_language_code) NOT IN (' + x[:-2] + ')'
		
		q += ' AND event_id BETWEEN 0 AND 203018'
		q += ' ORDER BY event_id ASC'

		c = db.execute(q, *exclude_lang)
		r = c.fetchone()
		cmd = """
			INSERT OR REPLACE INTO T_Translated values(?, ?, ?)
			"""
		while r:
			#print r
			cc = self.execute(cmd, r['event_id'], lang, int(time.time()))
			r = c.fetchone()

		self.conn.commit()
				
	def events(self, exclude_lang=()):
		# LIMIT 100 OFFSET 50 == LIMIT 50, 100

		q = 'SELECT * FROM T_Data WHERE event_id not in (SELECT event_id FROM T_Translated)'
		if len(exclude_lang):
			x = "?, " * len(exclude_lang)
			q += 'AND LOWER(iso_639_language_code) NOT IN (' + x[:-2] + ')'

		c = self.execute(q, *exclude_lang)
		r = c.fetchone()
		i = 0
		while r:
			#if i == 1:
			#	raise StopIteration
			event_id = r[0]
			event = self.getEvent(event_id)
			yield event
			r = c.fetchone()
			i += 1
		

def translate(event, gtrans):
	_dict = {
	 'en': 'eng',
	 'de': 'ger',
	}
	
	tevent = EPGDBEvent(event.id, _dict[gtrans.dstlang])
	tevent.title = gtrans.translate(event.title)
	tevent.sdescr = gtrans.translate(event.sdescr)
	tevent.edescr = gtrans.translate(event.edescr)

	return tevent


def epgdb_translate(db, lang, langs_exclude):	
	lang = ''
	gtrans = None
	i = 0
	for event in db.events(langs_exclude):
		if lang != event.lang:
			lang = event.lang
			gtrans = GTranslator('en', lang)
		
		#print '*' * 80
		print '>', event
		#print '-' * 80
		event = translate(event, gtrans)
		#print '<', event.id
		#print '*' * 80
		
		db.setEvent(event)

		time.sleep(DELAY_SECS)
		i += 1
	
	return i

def main():
	print("Copying '%s' to '%s'" % (DB_SRC, DB_TMP))
	fcopy(DB_SRC, DB_TMP)
	
	print("Translating epg data...")
	db = EPGDB(DB_TMP)
	db.initTableTranslated()
	#testing
	#db.markTranslated(TARGET_LANG, LANGS_EXCLUDE)
	r = epgdb_translate(db, TARGET_LANG, LANGS_EXCLUDE)
	print("Translated %d events." % r)
	
	cmd = """
	systemctl stop enigma2 &&
	cp %s %s &&
	systemctl start enigma2
	""" % (DB_TMP, DB_SRC)
	
	if r > 0 and RESTART_GUI:
		print("Restarting e2 GUI")
		print(cmd)
		os.system(cmd)
	

if __name__ == "__main__":
	main()
